// Based on the json-with-bigint library: https://github.com/Ivan-Korolenko/json-with-bigint/blob/main/json-with-bigint.js
// Sadly we can't use it directly: https://github.com/Ivan-Korolenko/json-with-bigint/issues/15

/* eslint-disable */
const noiseValue = /^-?\d+n+$/; // Noise - strings that match the custom format before being converted to it
export const JSONParse = (json: any) => {
    if (!json) return JSON.parse(json);
  
    const MAX_INT = Number.MAX_SAFE_INTEGER.toString();
    const MAX_DIGITS = MAX_INT.length;
    const stringsOrLargeNumbers =
      /"(?:\\.|[^"])*"|-?(0|[1-9][0-9]*)(\.[0-9]+)?([eE][+-]?[0-9]+)?/g;
    const noiseValueWithQuotes = /^"-?\d+n+"$/; // Noise - strings that match the custom format before being converted to it
    const customFormat = /^-?\d+n$/;
  
    // Find and mark big numbers with "n"
    const serializedData = json.replace(
      stringsOrLargeNumbers,
      (text: any, digits: any, fractional: any, exponential: any) => {
        const isString = text[0] === '"';
        const isNoise = isString && Boolean(text.match(noiseValueWithQuotes));
  
        if (isNoise) return text.substring(0, text.length - 1) + 'n"'; // Mark noise values with additional "n" to offset the deletion of one "n" during the processing
  
        const isFractionalOrExponential = fractional || exponential;
        const isLessThanMaxSafeInt =
          digits &&
          (digits.length < MAX_DIGITS ||
            (digits.length === MAX_DIGITS && digits <= MAX_INT)); // With a fixed number of digits, we can correctly use lexicographical comparison to do a numeric comparison
  
        if (isString || isFractionalOrExponential || isLessThanMaxSafeInt)
          return text;
  
        return '"' + text + 'n"';
      }
    );
  
    // Convert marked big numbers to BigInt
    return JSON.parse(serializedData, (_, value) => {
      const isCustomFormatBigInt =
        typeof value === "string" && Boolean(value.match(customFormat));
  
      if (isCustomFormatBigInt)
        return BigInt(value.substring(0, value.length - 1));
  
      const isNoiseValue =
        typeof value === "string" && Boolean(value.match(noiseValue));
  
      if (isNoiseValue) return value.substring(0, value.length - 1); // Remove one "n" off the end of the noisy string
  
      return value;
    });
};
